function eigsenc = eigsymencrohn(iA)
%BEGINDOC==================================================================
% .Author.
%
%  Jaroslav Horacek
%
%--------------------------------------------------------------------------
% .Description.
%
%  Very fast outer approximation of the eigenvalue interval bounds for a
%  real symmetric interval matrix
%
%--------------------------------------------------------------------------
% .Input parameters.
%
%  iA ........ a real symmetric interval matrix
%
%--------------------------------------------------------------------------
% .Output parameters.
%
%  ienc ... an interval vector containing an outer approximation of the 
%  eigenvalues of iA
%
%--------------------------------------------------------------------------
% .Implementation details.
%
%  The function computes its results by trivially following the statement
%  of the theorem. Both sets of eigenvalues, 'eigsAc' and 'eigsAd' must al-
%  ways be computed, even if we're looking only for the largest eigen-
%  value's upper bound, and the computation of the upper and lower bounds
%  takes very little time (it only involves adding and substracting a given
%  vector from another), so it is always performed.
%
%  If the function is called repeatedly from a more complex algorithm,
%  chances are that the verified eigenvalues of mid(iA) and rad(iA) will
%  need to be computed for several different functions; therefore, we
%  allow the caller to provide pre-computed results so that we avoid
%  redunadant computations.
%
%  method vereigsym returns eigenvalues sorted l1 <= l2 <= ...
%
%--------------------------------------------------------------------------
% .License
%
%  [License goes here]
%
%--------------------------------------------------------------------------
% .History
%
%  2018-1-13   first version
%
%--------------------------------------------------------------------------
% .Todo.
%
%
%ENDDOC====================================================================

   ieigsAc = vereigsym(mid(iA));
   ieigsAdelta = vereigsym(rad(iA));
   
   specrad = max( mag(ieigsAdelta)  );
   
   eigsenc = infsup( inf(ieigsAc) - specrad, sup(ieigsAc) + specrad );



end