function [iX, state] = ibacksubst(iA, iB) 
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
% function provides the backward substitution on the interval system
% given by matrix iA in REF and right hand sides vector(s) iB
%
% b is an interval vector
% works also if b is a matrix, if so a matrix of solution for every
% column vector in b is returned
%
% for overdetermined systems we first compute all intersections of variable
% x_n than we continue with the normal backward substitution
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ... input matrix in REF  
%   iB ... possible right hand side of system Ax=b, supports multiple
%   righthand sides [OPTIONAL]
%
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   iX ... enclosure of solution of Ax=b
%   state is a string flag explaining the type of returned solution
%    'inf' - infinite
%    'non' - no solution
%    'vec' - interval vector
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   NOTE - 'sin' can also mean, that all possible pivots contain 0 because 
%   of interval widening caused by interval operations
%   'inf' can also mean, that intersection of all equations containing
%   x_n during backward substitution is unbouded because of interval widening
%   caused by interval operations
%
%------------------------------------------------------------------------
%  .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2011-05-20    first version
%   2011-07-12    works for point real matrix
%   2018-02-13    input matrix shaped changed to plain REF
%
%------------------------------------------------------------------------
% .Todo.
%   
%   Make work for real matrices
%
%ENDDOC===================================================================

state = 'vec';
[m, n] = size(iA);

if( n > m )
    error( 'backsubst: wrong matrix size - m must be >= n' );
end 


% TODO !!!!
%[A, b] = tointval(A, b);
bsize = size(iB, 2); 

iX = infsupdec( zeros(n, bsize) );

% handling x_n for overdetermined system
ixn = repmat( infsupdec(-Inf, Inf), 1, bsize );

ixn = intersect( iB(n:m, :) ./ iA(n:m,n));

if (any(isempty(ixn)))
    state = 'non';
    iX = repmat(NaN, n, bsize);
    return;
else
  iX(n, :) = ixn;    
end

% backward substitution of other variables
for k=(n-1):-1:1
  iX(k, :) = ( iB(k,:) - iA(k, (k+1):n) * iX( (k+1):n,:) ) ./ iA(k, k);
end


%if(m > n)    
%    % compute enclosure of xn for each right-hand side
%    for j=1:bsize
%      righthand = iB(n:m, j) ./ iA(n:m, n);
%      ixn(j) = intersect( righthand);
%        
%      % empty intersection
%      if( isempty (ixn(j)) )
%         % error('Back substitution: empty intersection - system has no solution' );
%         iX = repmat(NaN, n, bsize );
%         state = 'non';
%         return;
%       end                
%    end
%else
%   ixn = iB(n, :) / iA(n, n);  
%end


%% test if xn is finite
%if(any(isinf(inf(ixn)) || isinf(sup(ixn)))  )
%    % error('Back substitution: infinite intersection - system solution not bounded');
%    state = 'inf';
%    iX = repmat( infsup(-Inf, Inf), n, bsize);
%    return;
%    
%else      
%    iX(n, :) = ixn;
%end

% backward substitution for x_(n-1) ... x_1
%for k=(n-1):-1:1
%    for j=1:bsize
%        iX( k, j ) = ( iB(k, j) - sum(  iA(k, (k+1):n) * iX( (k+1):n, j ) ) ) ;
%    end
%end





end