
function [iA, iB, state, d] = ige(iA, iB = [], pre=0)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   Gaussian elimination
%
%   Function provides Gaussian elimination on an interval matrix [A|B],
%   that represents system AX = B. possibly for multiple right hand side B
%   When ige(iA) is called the matrix A is elimintaed to REF 
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  m x n matrix 
%   iB ...  m x k matrix - optional
%   pre ... if nonzero or 1 means preconditioning is called, otherwise 0
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   iA  ...  m x n upper diagonal matrix, eliminated A matrix.
%   iB  ...  m x k matrix of eliminated right hand sides
%   d   ...  the sign of a determinant of given matrix
%   state...  'sin' if matrix seems to be singular, 
%             'ref' if REF is returned
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   We choose pivot for every column as the biggest mignitude
%   of an interval
%   Preconditioning is done with midpoint inverse in case of square 
%   systems/matrix.
%   In case of overdetermined system we use 
%   P =     | A_c1    0  | 
%           | A_c2 I(m-n)|
%   where A_c1 consists of first n rows of A_c, A_c2 is the rest
%
%------------------------------------------------------------------------
% .License.
%    Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.   
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-02-13    first version
%
%------------------------------------------------------------------------
% .Todo.
%   Make work for real matrices
%
%ENDDOC===================================================================




% handling input parameters
%if (nargin == 2) 
%  pre = 0;
%elseif (nargin == 1)
%  pre = 0;
%  iB = [];
%end


bsize = size(iB,2);

[m, n] = size(iA);


% TODO !!!!!!!
%[iA, iB] = tointval(A, b);

% initialization
state = 'ref';
d = 1;


% 1) preconditioning
if ( pre )
  [iA, iB] = ilsprecondinv(iA, iB);
end

% 2) creating extended system 
iC = [iA iB];



% repeating (n-1)-times
for k=1:(n-1)
    
    % 3) finding nonzero pivot
    % with greatest magnitude
    kcolumn = iC(k:m,k);
    inzero = in(0, kcolumn);
    kcolumn(inzero) = infsupdec(0);
    
    [ maxmig, index ] = max( mig( kcolumn ) );
    index = index + k - 1;
    
    % if mignitude is less or equal 0
    % A might be singular => every interval in k-th column contains 0
    if( maxmig <= 0)
        state = 'sin';
        return;
    end
    
    
    % 3) swapping i-th and k-th row
    if (k ~= index)
        iC( [k index] , : ) = iC( [index k] , : );
        d = -d;
    end
    
    
    % 4) division of k-th row with a_kk
    % on the position kk there remains interval [1,1]
    
%    akk = A(k, k);
%    A( k, (k+1):(n+bsize) ) = A(k, (k+1):(n+bsize)) / akk;
%    A(k, k) = intval(1);

   
    % 4) for every i-th row, i > k eliminate 
    % coefs under pivot    
    for j=(k+1):m
        ckk = iC(k, k);
        cjk = iC(j, k);
        iC(j, (k+1):(n+bsize) ) = iC(j, (k+1):(n+bsize)) -  (cjk / ckk ) * iC( k, (k+1):(n+bsize) );
        iC(j, k) = infsupdec(0);               % zeros under the pivot
    end
    
end

% 5) for overdetermined systems we continue
% by normalizing left equations 
% into shape [1,1] x_n = [a,b] 
%for i=n:m
%    iC(i, (n+1):(n+bsize) ) = iC(i, (n+1):(n+bsize)) / iC(i, n);
%    iC(i, n) = intval(1);
%end
    
iA = iC(:, 1:n);
iB = iC(:, (n+1):(n+bsize));

end