function [ix, state] = ilshbrenc(iA, ib)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   Computes Hansen-Bliek-Rohn enclosure of square linear system
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  m x n matrix, if the line is very long then we can
%          wrap it like this 
%   ib ...  n-dimensional right-hand side vector
%   pre ... parameter for preconditioning if 1 we precondition with 
%   inverse of Ac (midpoint matrix of A)
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   ix ...  enclosure if the solution of system iA x = ib
%   state ... contains additional info
%   'vec' if enclosure is finite or unbounded vector
%   'non' system has no solution if the estimation was correct
%   'fop' - preconditioning failed - use different method
%   'hul' - returned enclosure is hull
%   'nsq' - nonsquare matrix
%   'sng' - Ac matrix might be singular
%   'noth' - iA after preconditioning is not an H-matrix
%   ver ... 1 if result verified, 0 otherwise
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   If Ac is diagonal matrix with positive numbers, then HBR returns 
%   interval hull  
%
%------------------------------------------------------------------------
%  .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2011-11-12    first version
%   2015-04-24    applying template coding conventions  
%
%------------------------------------------------------------------------
% .Todo.
%   
%   If inverse of Ac does not exist use approximate inverse
%   Verified checking of spectral radius
%   - nahradit pinv
%
%ENDDOC===================================================================

ver = 1;
state = 'vec';

%iA = tointval(iA);
%ib = tointval(ib);

[m,n] = size(iA);

% ix inicialized wih [-Inf, Inf]
ix = repmat( infsupdec(-Inf, Inf), n, 1);

% iA must be a square matrix
if( m > n )
    state = 'nsq';
    ver = 1;
    return;
end

%if ( pre == 1 )
%   [iA,ib, state] = ilsprecondinv(iA,ib); 
%   if(strcmp(state, 'fop'))
%       ver = 0;
%       return;      
%   end
%end

[iA,ib] = ilsprecondinv(iA,ib);

Ac = mid(iA);
Adelta = rad(iA);
bc = mid(ib);
bdelta = rad(ib);


% matrix Ac might be regular
%if( rank(Ac) < n)
%    ver = 0;
%    state = 'sng';
%    return;
%end

% test if spectral radius of |inv(Ac)| * Adelta is < 1
% testování H-maticovosti?
   
if(~ishmatrix(iA) )  
    ver = 0;    
    state = 'noth';
    return; 
end

% main body of computation of hbr enclosure
xc = Ac \ bc;
M = pinv( eye(n) - abs( pinv(Ac)) * Adelta );
xstar = M*( abs(xc) + abs( pinv(Ac))*bdelta );

for i=1:n
    ximin = min( -xstar(i) + M(i,i)*( xc(i) + abs(xc(i))), 1/(2*M(i,i) - 1)*(-xstar(i) + M(i,i)*(xc(i) + abs(xc(i)))));    
    ximax = max( xstar(i) + M(i,i)*( xc(i) - abs(xc(i))), 1/(2*M(i,i) - 1)*(xstar(i) + M(i,i)*(xc(i) - abs(xc(i)))));
    
    ix(i) = infsupdec(ximin, ximax);
end
    


