function [ihull, state, ver] = ilshull(iA, ib)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   computes hull of iA x = ib 
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  interval or real matrix 
%   ib ... interval right hand side
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   ihull ... hull of the ILS solution set 
%   state ... state 'vec' - vector returned, 'emp' - some intervals empty, 
%             'inf' - some coefficients have infinite bounds
%   ver ... 1 if the result is verified, otherwise 0
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%------------------------------------------------------------------------
% .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-07-15    first version
%   2018-10-17    mistake with wrong ul, ub corrected
%   2019-03-11    flags added
%
%------------------------------------------------------------------------
% .Todo.
%
%  - rework initial enclosure
%  - add verification to linear programming
%
%ENDDOC===================================================================


ver = 0;
state = 'vec';

[m, n] = size(iA);
%[A, b] = tointval(iA, b);
    
maxx = repmat(-Inf, n, 1);
minx = repmat(Inf, n, 1);

Ac = mid(iA);
Delta = rad(iA);
bc = mid(ib);
delta = rad(ib);

if (m==n)
  ix = ilshbrenc(iA, ib);
end

if (m > n)
  ix = oilsrohnenc(iA, ib);
end

sign = intsign(ix);

sign = sign';
    
% add exception    
    
    
% set of all orthants to search: orthx 
zerospos = find( sign == 0 );
zeronum = length(zerospos);
Yn = generateyn(zeronum);

% sign vector of all orthants that need to be checked
len = 2^zeronum;
orthx = repmat( sign, len, 1 );
orthx(:, zerospos) = Yn;



I = eye(n);
ctype = blanks(2*m + 1);
ctype(1:m) = 'U';
ctype((m+1):(2*m+1)) = 'L';
vtype = [];
vtype = repmat('C', n, 1);
lb = inf(ix);
ub = sup(ix);


for i=1:len
    y = orthx(i, :);
        
    % computing enclosure by solving linear programming method
    % in orthant defined by code y - rewritten Oettli-Prager theorem for one orthant
    Dy = diag(y);
   
    DeltaDy = Delta * Dy;
    C = [ Ac - DeltaDy ; Ac + DeltaDy; y];
    d = [ sup(ib); inf(ib); 0];
    
    % computing for each variable max and min
    for j=1:n
        
        % j-th column of eye matrix
        ej = I(:, j);
        
        % computing minimum
        [x, fmin, status, extra] = glpk(ej, C, d, lb, ub, ctype);

        if (status==0 &&  x(j) < minx(j))
          minx(j) = x(j);
        end
        if (status==11)
          minx(j) = -Inf;
        end
        
        % computing maximum
        [x, fmax, status, extra] = glpk(ej, C, d, lb, ub, ctype, vtype, -1);
        if (status == 0 && x(j) > maxx(j))
          maxx(j) = x(j);
        end
        if (status==11)
          maxx(j) = Inf;
        end
            
    end        
end
    
%if( nosolution )
%   sol = 'non'; 
%   return; 
%end
    
%if (any(isnan(max)) || any(isnan(min)) || any(isfinite(max) == 0) || any( isfinite(min) ==0 ) )
%    sol = 'non';
%    x = repmat(intval(NaN), n, 1);
%    return;
%end


if any(maxx < minx)
   % the bounds remained unchanged, upper bound is lower than lower bound
   %  => result is empty interval 
  ihull = empty(m, 1);
  state = 'emp';      
elseif all(isfinite([minx; maxx]))
   state = 'vec';
   ihull = infsup(minx, maxx);
else
   state = 'inf';
   ihull = repmat(infsupdec(-Inf, Inf), m, 1);
end
  

   

end

