function [ix, state, it] = ilsinflationenc(iA, ib, maxit)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   computes an enclosure of a system iA x = ib using Rump's inflation method 
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  m x n matrix, if the line is very long then we can
%          wrap it like this 
%   ib ...  n-dimensional right-hand side vector
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   ix ...  solution of system iA x = ib
%   state ... contains additional info 
%   'vec' if enclosure is finite or unbounded vector
%   'exc' iterations exceeded
%   'dif' - Krawczyk method cannot be used - use different method
%   it ... number of iterations done 
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   Based on paper Rump, Kaucher, Small Bounds for the Solution of 
%   Systems of Linear Equations, 1980
%
%   works when iA is strongly regular
%
%------------------------------------------------------------------------
%  .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-09-30    first version
%
%------------------------------------------------------------------------
% .Todo.
%   
%
%ENDDOC===================================================================


state = 'vec';
%[A, b] = tointval(A, b);
[m, n] = size(iA);


if (m != n)
  error('ilsinflationenc: matrix iA must be n x n');  
end

if nargin < 3
  maxit = 10;
end  
  

it = 1;

Ac = mid(iA);
bc = mid(ib);
C = inv(Ac);
I = eye(n);

iR = (I - C * iA);

if (imaxnorm(iR) >= 1)
  ix = repmat(NaN, n, 1);
  state = 'dif'; 
  return;
end

xc = C * bc;
yc = C * (bc - Ac * xc); 

iy0 = yc + C*( ib - iA*xc - iA * yc);
iyk = iy0;
e = 0.1;
k = -1;

notin = 1;
it = 0;
while (notin && it <= maxit)
  it = it + 1
  iyk = midrad( mid(iyk), (1 + e)* rad(iyk) );
  newiyk = iy0 + iR * (iyk - yc) ;
  if (in (newiyk, iyk))
    notin = 0;
  end  
  
  iyk = newiyk;
end

% iterations num exceeded
if (it > maxit)
  state = 'exc';  
end

ix = iyk + xc;



end