function [ix, state] = ilsjacobistep(iA, ib, ix)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   provides one step of interval Jacobi iteration method
%   works for square and overdetermined systems
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  m x n matrix, if the line is very long then we can
%          wrap it like this 
%   ib ...  n-dimensional right-hand side vector
%   ix ... initial enclosure of solution
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   ix ...  shaved ix using Jacobi iteration on iA x = ib
%   state ... contains additional info 
%   'vec' if enclosure is finite or unbounded vector
%   'non' system has no solution inside ix
%   'nan' zeros on diagonal or too large intervals
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   if iA is rectangular m x n matrix, only first n rows are used for
%   iteration
%
%   to ensure speed it does not check whether elements on diagonal are nonzero!
%   no preconditioning is used
%   the implementation is like this to enable its use as a contractor in nils
%------------------------------------------------------------------------
%  .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2015-04-24      first version
%   2018-09-25      ommitted checking diagonal for zeros
%
%------------------------------------------------------------------------
% .Todo.
%  
%  Maybe skip checking tointval and zeros on diagonal to ensure speed
%
%ENDDOC===================================================================


state = 'vec';
ioldx = ix;

[m, n] = size(iA);


%[iA,ib] = tointval(iA,ib);


% checking if a diagonal contains 0
%for j=1:n
%   if( in(0, iA(j, j) )  )
%       % zero on diagonal
%       state = 'zer';
%       return;
%   end
%end


iD = diag( 1./diag(iA) );
iJ = tril(iA, -1) + triu(iA, 1);
ix = iD *(ib - iJ*ix);
%ix = intersect(ix, ioldx);


% one Jacobi iteration step


% checking whether new enclosure is finite 
%if (any (isnan(inf(ix))) || any(isnan(sup(ix))))
%  state = 'nan';
%  return; 
%end

% intersection with old enclosure
ix = intersect( ix, ioldx );
if( any( isempty(ix) ) )
   % wrong first enclosure estimation
   state = 'non';
   return;
end


    
end