function [ix, state, it] = ilskrawczykenc(iA, ib, e, maxit, ioldx)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   computes an enclosure of a system iA x = ib using Krawczyk iteration 
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  m x n matrix, if the line is very long then we can
%          wrap it like this 
%   ib ...  n-dimensional right-hand side vector
%   e ... epsilon finish condition
%   maxit ... max number of iterations (default 20)
%   ioldx ... initial solution enclosure
%   zeronum ... first zeronum elements of enclosure estimation are set [0,0] 
%  (default 0)
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   ix ...  solution of system iA x = ib
%   state ... contains additional info 
%   'vec' if enclosure is finite or unbounded vector
%   'exc' iterations exceeded
%   'dif' - Krawczyk method cannot be used - use different method
%   'nan' - nan returned
%   it ... number of iterations done 
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   for start we estimate enclosure using ilskrawczykest
%
%   we stop when the absolute differences between all infs and all sups of
%   the two consequent solutions are less than positive real number e 
%   (default min(min(diam(A))) * 10^-5
%   10^(-10) added to work for point systems with default e
%
%   !!! method can be used only if inv(Ac) iA is H-matrix
%
%------------------------------------------------------------------------
%  .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2011-07-12    first version
%   2015-05-18    applying template coding conventions  
%   2018-09-26    zeronum discarded, flags changed
%
%------------------------------------------------------------------------
% .Todo.
%   
%
%ENDDOC===================================================================


state = 'vec';
%[A, b] = tointval(A, b);
[m, n] = size(iA);
it = 1;




%[iA, ib] = ilsprecondinv(iA, ib);



% default e, maxit, zeronum
if( nargin < 3)
   e = min(min(diam(iA))) * 10^(-5);
   maxit = 20; 


   [ioldx, state] = initkrawczykenc(iA, ib); 
   ix = ioldx;

    % krawczyk estimation failed - Krawczyk method cannot be used
    if( strcmp(state, 'dif'))
        state = 'dif';
        return;
    end
end


iE = iA - eye(m); 

while(it <= maxit)
    
    ix = intersect( ib - iE * ioldx, ioldx );
    
    % intersection with old enclosure
    ix = intersect( ix, ioldx );
    if( any( isempty(ix) ) )
      % wrong first enclosure estimation
      state = 'empty';
    return;
end
    
    
    % stopping condition
    if ( all(abs(inf(ix)-inf(ioldx)) < e) && all(abs(sup(ix) - sup(ioldx)) < e ) )
        return;
    end
    
    it = it + 1;
    ioldx = ix;
    
end

% checking whether new enclosure is finite 
if (any (isnan(inf(ix))) || any(isnan(sup(ix))))
   state = 'nan';
   return; 
end

% number if iterations exceeded
if( it > maxit )
    state = 'exc';
end


end