function [ix, state, it] = vsol(A, b, itnum)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   verified solution of a real linear system, works for square systems
%  works also for multiple right hand sides
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  n x n matrix, if the line is very long then we can
%          wrap it like this 
%   ib ...  n-dimensional right-hand side vector
%   itnum ... number of iterations (random subsystems chosen) 
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   ix ... enclosure of the solution vector
%   state ... 'vec' interval vector, 'dif' use different method
%
%------------------------------------------------------------------------
% .Implementation details. 
%  
%   based on Rump's inflation algorithm 
%   Siegfried Rump, Verification methods: Rigorous results using floating-point
%   arithmetic, 2010 
%
%------------------------------------------------------------------------
%  .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-10-08      first version
%
%------------------------------------------------------------------------
% .Todo.
%   
%  correst for multiple right-hand sides 
%
%ENDDOC===================================================================

iA = infsupdec(A);
ib = infsupdec(b);

[m, n] = size(A);


if (m != n)
  error('vsol: input matrix must be n x n');  
end

if (nargin < 3)
  itnum  = 15;
end
  
state = 'vec';  
ix = repmat(infsup(-Inf, Inf), n, 1);

iR = infsupdec(inv(mid(iA)));
xc = iR * mid(ib);
iC = eye(n) - iR * infsupdec(iA);
iZ = iR * (ib - iA * infsupdec(xc));
iX = iZ;
it = 0;

while (it < itnum)
  it = it + 1;
  iY = iX * infsupdec(0.9, 1.1) + 1e-20*infsupdec(-1,1);
  iX = iZ + iC * iY;
  if all (interior( iX, iY))
    ix = xc + iX;
    return;
  end
   
end

  
    
end