function [inviA, state, ver] = imatinv(iA)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   computes enclosure of inverse matrix to iA
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  interval or real matrix 
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   invA ...  inverse matrix
%   ver ... 1 if result is verified, otherwise 0
%   state ... 'reg' - matrix regular, 'sin' - matrix singular, 'unk' - 
%            singularity or regularity not verified
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%  if both bounds are inverse nonnegative -> apply Kuttler's theorem
%  see Neumaier, Interval methods for systems of equations, 1990.  
%
%  uses HBR method
%  if Ac matrix of iA is diagonal, HBR returns inverse hull
%
%
%------------------------------------------------------------------------
% .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2019-03-31    first version
%
%------------------------------------------------------------------------
% .Todo.
%   
%
%ENDDOC===================================================================


[m, n] = size(iA);

if (m != n)
  error('imatinv: matrix must be n x n');  
end


inviA = repmat(NaN, n, n);
ver = 0;



% checking regularity of iA
[issin, state, ver] = issingular(iA);
if (issin && ver)
   return;  
end


[isreg, state, ver] = isregular(iA);
if (ver==0)
    state = 'unk';
    return  
end

if (isreg && ver)


  iI = infsupdec(eye(n));
  
  % verified inverses of bounds of iA
  uAinv = vinv(inf(iA));
  oAinv = vinv(sup(iA));
  
  % if possible -> apply Kuttler's theorem
  if (all(all(uAinv >=0)) && all(all(oAinv>=0)))
    inviA = infsupdec(sup(oAinv), inf(uAinv));
    ver = 1;
    return;
  end
  
  % otherwise use enclosure computation for ILS
  inviA = [];
    for i=1:n
      b = iI(1:n, i);
      ix = ilshbrenc(iA, b);
      inviA = [inviA, ix];
    end
end    

end