function [ism, u, state] = ismmatrix(iA, checkminors)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   decides whether matrix iA is an M-matrix
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  interval or real matrix 
%   checkminors ... 0/1, 1 by default
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   ism ...  1 if matrix is M-matrixf  
%   u ... vector u>0 witnessing that iA is a M-matrix
%   state ... 'vec' found u
%             'nou' could not find u or M-matrix verified using principal minors
%             'bad' iA is not a Z-matrix 
%             
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%  conditions based on chapter 3.6 from A. Neumaier, Interval methods for 
%   systems of equations, 1990  
%
%  checking all principal leading minors for positivity can be time consuming 
%
%------------------------------------------------------------------------
% .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-09-20    first version!
%   2018-09-24    added u, state
%   2018-09-24    computation of principal leading minors added
%
%------------------------------------------------------------------------
% .Todo.
%
%ENDDOC===================================================================


[m, n] = size(iA);

ism = 0;
shapeok = 0;
state = 'nou';
u = zeros(n, 1);


if (m != n)
  error('ismmatrix: matrix must be n x n ');
end  


if (nargin < 2)
    checkminors = 1;  
end

% first checking that Aij < 0 for i != j
supd = abs(sup(diag(iA))) + 1;
D = sup(iA) - eye(m) * supd;
if (all (all (D < 0)))
  shapeok = 1;
else 
    state = 'bad';
end
    

% try to find u>0 such that Au>0 
if (shapeok)
  C = inf(iA);
  d = ones(m, 1);
  u = C \ d;
  if (all (u > 0))
    if(all((iA * u) > 0)  ) % here is inf(iA) enough?
        ism = 1;
        state = 'vec';
    end 
  end
  
end


% checking leading principal minors for positivity
if (strcmp(state, 'nou') && checkminors)
  idets = zeros(n, 1);
  for i=1:n
      subiA = iA(1:i, 1:i);
      if (idethull(subiA) > 0)
         idets(i) = 1;   
      end 
  end
  
  if all(idets)
    ism = 1;  
  end
end



end