function [solution, splitnum, ctime] = inlsbab(initbox, f, J, epsilon, prunestrategy)  
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   Branch and Bound approach using interval linearization
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   initx ... initial framing box for B&B procedure
%   f ... anonymous multi-dimensional function
%   J ... Jacobian of that function
%   epsilon ... terminal size of boxes
%   prunestrategy ... selection strategy for vertice used for linearization
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   solution ... boxes that may contain a solution
%   splitnum ... number of box splittings 
%   ctime ... computational time
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   The boxes we splits into two boxes according to longest side. 
%   Based on interval linearization.  
%
%------------------------------------------------------------------------
% .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-11-28    first version
%
%------------------------------------------------------------------------
% .Todo.
%   
%
%ENDDOC===================================================================

% initboxes fills the stack with some boxes (interval column vector) to prune
 
%  initxc = num2cell(initx);
  [m, n] = size(J);
  
  solution =  [];
  ctime = 0;
  tic;
  splitnum = 0;  

  % column vectors
  stack = initbox;
 
  lastboxindex = 1;
  

  while (lastboxindex > 0)
      newbox = stack(:, lastboxindex);
      
      printf("size of box: %d x %d \n", wid(newbox(1)), wid(newbox(2)));
      fflush(stdout);
      
      
      % hull consistency 
  
      
      % pruning 
      [newbox, empty] = inlsprune_lp(newbox, f, J, prunestrategy);
  
  
      % empty box   
      if (empty)
          lastboxindex = lastboxindex - 1;
          continue; 
      end       
   
      % box too small and zero probably there
      if toosmalltest(newbox, epsilon)
           % row vectors!!!!     
           %solution = [solution, stack(:,lastboxindex)];
           solution = [solution, newbox];
           lastboxindex = lastboxindex - 1;
           continue;
      end    

     
     [b1, b2] = splitbox(newbox);
     splitnum = splitnum + 1;
     stack(:, lastboxindex) = b1;  
     stack(:, lastboxindex + 1) = b2;
     
     lastboxindex = lastboxindex + 1;    
   
  end

 ctime = toc;
  

end



function [newbox, empty] = inlsprune_lp(box, f, J, prunestrategy)
  
  len = length(box);
  boxc = num2cell(box);
  [m, n] = size(J);
  iA = infsup(zeros(m, m));
  empty = 0;
  
  % strange trick to evaluate jacobian, lie after line
  for i=1:m
    fh = function_handle(J(i, :));
    iA(i, :) = fh(boxc{:});
  end
  
  Ac = mid(iA);
  Arad = rad(iA);
    
  range = box;
  
  % midpoint selection process
   switch prunestrategy 
   case 'm' 
        x0 = [mid(box)];
   case 'c'
        l = length(box);
        k = randi([0 1],l, 1);
        boxinf = inf(box);
        boxdiam = diam(box);
        x0 = [ boxinf + k .* boxdiam];    
   case 'cc'
        l = length(box);
        k = randi([0 1],l, 1);
        kinv = ~k;
        boxinf = inf(box);
        boxdiam = diam(box);
        x0 = [ boxinf + k .* boxdiam , boxinf + kinv .* boxdiam ];
    case 'i'
        % box 1/3 and 2/3 of each side
        box = infsup( inf(box) + 1/3 * diam(box), inf(box) + 2/3 * diam(box) );
        l = length(box);
        k = randi([0 1],l, 1);
        boxinf = inf(box);
        boxdiam = diam(box);
        x0 = [ boxinf + k .* boxdiam];    
    case 'ii'
        % box 1/3 and 2/3 of each side
        box = infsup( inf(box) + 1/3 * diam(box), inf(box) + 2/3 * diam(box) );
        l = length(box);
        k = randi([0 1],l, 1);
        kinv = ~k;
        boxinf = inf(box);
        boxdiam = diam(box);
        x0 = [ boxinf + k .* boxdiam , boxinf + kinv .* boxdiam ];
    case 'iim'
        % box 1/3 and 2/3 of each side
        box = infsup( inf(box) + 1/3 * diam(box), inf(box) + 2/3 * diam(box) );
        l = length(box);
        k = randi([0 1],l, 1);
        kinv = ~k;
        boxinf = inf(box);
        boxdiam = diam(box);
        x0 = [ mid(box), boxinf + k .* boxdiam , boxinf + kinv .* boxdiam ];
    case 'ccm'
        l = length(box);
        k = randi([0 1],l, 1);
        kinv = ~k;
        boxinf = inf(box);
        boxdiam = diam(box);
        x0 = [ mid(box), boxinf + k .* boxdiam , boxinf + kinv .* boxdiam];
    case 'ccii'
        l = length(box);
        k = randi([0 1],l, 1);
        kinv = ~k;
        boxinf = inf(box);
        boxdiam = diam(box);
        x0 = [ boxinf + k .* boxdiam , boxinf + kinv .* boxdiam ];
         % box 1/3 and 2/3 of each side
        box = infsup( inf(box) + 1/3 * diam(box), inf(box) + 2/3 * diam(box) );
        l = length(box);
        k = randi([0 1],l, 1);
        kinv = ~k;
        boxinf = inf(box);
        boxdiam = diam(box);
        x0 = [ x0, boxinf + k .* boxdiam , boxinf + kinv .* boxdiam ];  
    case 'r'        
      x0 = [randfim(box)];    
    case 'rr'
        x0 = [randfim(box), randfim(box)];
    case 'rrr'
        x0 = [randfim(box), randfim(box), randfim(box)];
    case 'rrrr'
        x0 = [randfim(box), randfim(box), randfim(box), randfim(box)];
        
    otherwise
      error('inlsbab: pruning strategy with that name does not exist')     
    end
  
  
  % forming constraints for linear program
  lpA = [];
  lpb = [];
  mpnum = size(x0, 2);

  for k=1:mpnum
  
    midpoint = x0(:, k);
    midpointc = num2cell(midpoint);
   
    alpha = ( 1 ./ rad(range) ) .* ( mid(range) - midpoint );
    v0 = ( 1 ./ rad(range) ) .* (  mid(range) .* midpoint  - sup(range) .* inf(range) );
    
    lpA = [lpA; Ac - Arad * diag(alpha); -Ac - Arad * diag(alpha) ];
    lpb = [lpb; Ac * midpoint+ Arad*v0 - f(midpointc{:});  -Ac * midpoint+ Arad*v0 + f(midpointc{:}) ];
  end

  ctype = blanks(2*mpnum*m);
  ctype(1:(2*mpnum*m)) = 'U';
  vtype = [];
  vtype = repmat('C', n, 1);
  
  
  
  % solving linear program 2n times (each of n directions for min/max)
  maxbound = sup(box);
  minbound = inf(box);
  I = eye(n);
  
  for j=1:n 
      
    % j-th column of eye matrix
      ej = I(:, j);
      
      
      % computing minimum
      [xsol, fmin, err, extra] = glpk(ej, lpA, lpb, minbound, maxbound, ctype);

      
      if (err == 10)
          empty = 1;
          newbox = box;
          return;
      end
      
      if(err == 0 )
          if(xsol(j) > minbound(j))
              minbound(j) = xsol(j);
          end
      end
  
      % computing maximum
      [xsol, fmax, err, extra] = glpk(ej, lpA, lpb, minbound, maxbound, ctype, vtype, -1);

      
      if (err == 10)
          empty = 1;
          newbox = box;
          return;
      end
      
      if(err == 0)
          if(xsol(j) < maxbound(j))
              maxbound(j) = xsol(j);
          end
      end
      
  end
    

% getting bound together
newbox = infsup(minbound, maxbound);
  
  
end




% tests whether all sides of box are smaller than eps
function tsm = toosmalltest(box, epsilon)
    
    tsm = 0;    
    diams = wid(box);
    
    if (max(diams) < epsilon)
        tsm = 1;
    end  
    
end

% splits the box into two boxes according to longest side
function [b1, b2] = splitbox(box)
    
    diams = diam(box);
    % finds index of maximal diameter
   
    %i = randi(length(diams));
    [c, i] = max(diams);
    
    b1 = box;
    b2 = box;
    
    splitint = box(i);
    
    b1(i) = infsup(inf(splitint), mid(splitint));
    b2(i) = infsup(mid(splitint), sup(splitint));
    
end