function x = oilsrohnencit(A, b, e, it, maxit)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   returns an enclosure of a solution set of a system Ax = b using 
%   Rohn method
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   iA ...  m x n matrix
%   ib ...  n-dimensional vector
%   iterate ... 0 (default )direct method used for finding d, 1 - iterative 
%      search for d 
%   e ... small positive number (default min(min(diam(A))) * 10^(-5))
%   maxit defines maximal number of iteration steps
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   ix ... the enclosure of iAx=ib 
%   state ... a string flag
%       'vec' - x is interval vector
%       'dif' - if spectral radius of G is not < 1, use different method
%       'sin' - midpoint matrix might be singular
%       'exc' - iterations number exceeded
%       'nof' - direct method did not find d 
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%
%------------------------------------------------------------------------
%  .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2011-05-01    first version
%   2011-07-12    works for point real matrix
%   2015-05-11    applying template coding conventions 
%
%------------------------------------------------------------------------
% .Todo.
%   
%
%ENDDOC===================================================================

if( nargin < 6)
    maxit = 100;
end

[m,n] = size(A);
[A,b] = tointval(A,b);

itnum = 1;

while (itnum <= it)
    
    Delta = rad(A);
    delta = rad(b);

    pointA = randfim(A);
    pointb = randfim(b);
    
    % if A is singular we simply skip to the next iteration step
    if( rank( pointA ) ~= n )
        continue;
    end
    
    % R is almost Moore-Penrose inverse
    R = ((pointA')*pointA) \ (pointA');
    x0 = R * pointb;

    [G, g] = rohngg( R, x0, Delta, delta, pointA, pointb );
    
    % choosing method
    if( strcmp(method, 'dir') )
        d = rohnddir(G, g, e);
    elseif ( strcmp(method, 'ite') )
        d = rohndit(G, g, e, maxit);
    else
        error('rohnencit: wrong method name');
    end
    
    % provided only if d > 0
    if ( all( d > 0 ) )
        if (itnum == 1)
            x = midrad(x0, d);
        else
            x = intersect( x, midrad(x0, d)); 
        end    
    end
        
    itnum = itnum + 1;
end
      
end