function [solution, discarded, depth] = ilsbab(initx, eps, iA, ib)  
%BEGINDOC=================================================================
% .Author.
%
%  Adam Szabo, Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%
%   Branch and Bound on solution
%
%   The function applies Branch and Bound approach on a interval linear 
%   systems Ax = b (solution). 
%
%-------------------------------------------------------------------------
% .Input parameters.
%   
%   initx ... initial framing box before B&B procedure
%   vectors initial box(es) 
%   eps ... the minimum width of the interval that is allowed to divide
%   iA ... matrix of iA x = ib
%   ib ... right-hand side of of iA x = ib
%   typeofprune ... the name of method of prunes for boxes
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   solution ... boxes that may contain a solution
%   discarded ... boxes that certainly do not contain a solution 
%   depth ... determines the depth in which the box is discarded
%
%------------------------------------------------------------------------
% .Implementation details. 
%
%   The boxes we splits into two boxes according to longest side. 
%
%------------------------------------------------------------------------
% .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-01-02    first version
%
%------------------------------------------------------------------------
% .Todo.
%   
%   Added other methods of prunes for boxes.
%   Change the eighth input parameter (typeofprune) to the parameter of 
%   the function.   
%
%ENDDOC===================================================================

% initboxes fills the stack with some boxes (interval column vector) to prune
  [m, n] = size(initx);

  solution =  [infsup(1,2); infsup(1,2)];

  discarded = [infsup(1,2); infsup(1,2)]; 
  depth = [];

  % column vectors
  stack = initx;
  splitdepth = [0];
  lastboxindex = n;
 
  

  while (lastboxindex > 0)
      newbox = stack(:, lastboxindex);
      
      unf = 1;
      testint = iA*newbox - ib;
      if all( subset(0, testint) )
        unf = 0;
      end
      
      if (unf)
          discarded = [discarded, stack(:,lastboxindex)] 
          depth = [depth; splitdepth(lastboxindex)];
          %disp('ok')
          lastboxindex = lastboxindex - 1;
          continue; 
      end       
   
      % box too small and zero probably there
      if toosmalltest(newbox, eps)
           % row vectors!!!!     
           %solution = [solution, stack(:,lastboxindex)];
           solution = [solution, newbox];
           lastboxindex = lastboxindex - 1;
           continue;
      end    

     sd = splitdepth(lastboxindex); 
     
     [b1, b2] = splitbox(newbox);
     stack(:, lastboxindex) = b1;  
     stack(:, lastboxindex + 1) = b2;
     
     splitdepth(lastboxindex) = sd + 1;
     splitdepth(lastboxindex + 1) = sd + 1;
     
     lastboxindex = lastboxindex + 1;    
   
  end

  time = toc;
  [m, n] = size(solution);
  solution = solution(:, 2:n);
   [m, n] = size(discarded); 
  discarded = discarded(:, 2:n);
  

end

% creates interval matrix and vector from parametric systems 
function [iA, ib] = formmatrix(ipk, Ak, bk, A, b)
    
    iA = A;
    ib = b;
    
    if length(ipk) > 0
    
      for i=1:length(ipk)
        iA = iA + ipk(i) .* Ak{i};
        ib = ib + ipk(i) .* bk{i};
      end
    
    end
 
end

% tests whether all sides of box are smaller than eps
function tsm = toosmalltest(box, eps)
    
    tsm = 0;    
    diams = diam(box);
    
    if (max(diams) < eps)
        tsm = 1;
    end  
    
end

% splits the box into two boxes according to longest side
function [b1, b2] = splitbox(box)
    
    diams = diam(box);
    % finds index of maximal diameter
   
    %i = randi(length(diams));
    [c, i] = max(diams);
    
    b1 = box;
    b2 = box;
    
    splitint = box(i);
    
    b1(i) = infsup(inf(splitint), mid(splitint));
    b2(i) = infsup(mid(splitint), sup(splitint));
    
end