function [newboxes] = mergedivide(boxes, sizelimit)
%BEGINDOC=================================================================
% .Author.
%
%  Jaroslav Horacek
%
%-------------------------------------------------------------------------
% .Description.
%  
%  Bounds boxes obtained from B&B procedures from the outer recursive 
%  by quadrants while the size of quadrant fits the restrictive conditions.
%
%-------------------------------------------------------------------------
% .Input parameters.
%
%   boxes ... boxes containing the solution
%   limitsize ... maximum size of output box (otherwise recursive splitting) 
%   splitorth ... if true (default) splits area into orthants, otherwise larges 
%                 size is chosen and split to halves 
%
%------------------------------------------------------------------------
% .Output parameters.
%
%   newboxes ... output new boxes bounded by quadrants
%  
%------------------------------------------------------------------------
% .Implementation details. 
%  
%
%------------------------------------------------------------------------
% .Licence.
%  
%   Copyright (C) 2018  Charles University in Prague, Czech Republic
%
%   LIME 1.0 is free for private use and for purely academic purposes. 
%   It would be very kind from the future user of LIME 1.0 to give 
%   reference that this software package has been developed 
%   by at Charles University, Czech Republic.
%
%   For any other use of LIME 1.0 a license is required.
%
%   THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT ANY EXPRESS OR IMPLIED 
%   WARRANTIES, INCLUDING, WITHOUT LIMITATIONS, THE IMPLIED WARRANTIES 
%   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.    
%    
%-------------------------------------------------------------------------
% .History.
%       
%   2018-07-22    first version
%   2018-08-02    made faster on 400 boxes 50s -> 30s
%
%------------------------------------------------------------------------
% .Todo.
%    
%   implement splitting to halves
%
%ENDDOC===================================================================


  warning off;

  % splitting to orthants by default
%  if (nargin < 3)
%    splitorth = 1;  
%  end
  
  newboxes = [];
  
  stack{1} = boxes;
  stacksize = 1;
    
  while( stacksize > 0 )
    boxes = stack{stacksize};     
    boundbox = boundbybox(boxes);  
         
    if issmallenough(boundbox, sizelimit)
      newboxes = [newboxes, boundbox];
      stacksize = stacksize - 1;
    else  
      [list1, list2] = divideboxes(boundbox, boxes);
           
      ie1 = isempty(list1);
      ie2 = isempty(list2);
      
      if (ie1 && ie2)
        stacksize = stacksize - 1;
      end
      
      % both lists are not empty
      if (!ie1 && !ie2)
        stack{stacksize} = list1;
        stacksize = stacksize + 1;
        stack{stacksize} = list2;        
      else
      
        % only list1 nonempty
        if (!ie1 )
          stack{stacksize} = list2;
        end     
        
        % only list2 nonempty   
        if (!ie2 )
          stack{stacksize} = list1;
        end
      end        
    end
    
   
  end
  
  
  
end




% bounds boxes with one hull box
function hullx = boundbybox(boxes)
  [dim, len] = size(boxes);
  hullx = empty(dim, 1);
  %tic;
  for i=1:len
    box = boxes(:, i);
    hullx = union(hullx, box);
  end
   % printf("boundbybox: length: %d, time: %d \n", len, toc);
  %fflush(stdout);
  
end


% function splits box into parts and bound each part with a box
function [list1, list2] = divideboxes(boundbox, boxes)
  
  % splitting box according to its longest side
  [b1, b2] = splitbox(boundbox);
  
  [dim, len] = size(boxes);
  
  bounds = [];
  
  list1 = [];
  list2 = []; 
  
  %tic;
  
  for i=1:len
    ix = boxes(:, i);
    i1 = intersect(b1, ix);
    i2 = intersect(b2, ix);
    if ( all(isempty(i1) == 0) )
      list1 = [list1, i1];  
    end

    if ( all(isempty(i2) == 0) )
      list2 = [list2, i2];
    end     
  end

  %printf("divideboxes: length: %d, time: %d \n", len, toc);
  %fflush(stdout);
  
end


% splits the box into two boxes according to longest side
function [b1, b2] = splitbox(box)
    
    diams = diam(box);
    % finds index of maximal diameter
   
    %i = randi(length(diams));
    [c, i] = max(diams);
    
    b1 = box;
    b2 = box;
    
    splitint = box(i);
    
    b1(i) = infsup(inf(splitint), mid(splitint));
    b2(i) = infsup(mid(splitint), sup(splitint));
    
end


% function takes all boxes and checks whether they are too large
function ise = issmallenough(box, sizelimit)
  ise = 0;
  if (max( wid(box) ) < sizelimit)
    ise = 1;    
  end
end
